-- Author credits to Grant Fritchey from "Performance Tuning Distilled"

USE AdventureWorks2014
GO
IF  EXISTS (SELECT * FROM sys.indexes 
			WHERE object_id = OBJECT_ID(N'[Sales].[SalesOrderHeader]') 
				AND name = N'IX_SalesOrderHeader_OrderDate')
	DROP INDEX [IX_SalesOrderHeader_OrderDate] ON [Sales].[SalesOrderHeader] 
GO
CREATE INDEX IX_SalesOrderHeader_OrderDate ON Sales.SalesOrderHeader(OrderDate)
GO
SET NOCOUNT ON
GO

-----------------------------------------------------------------------------------------
--
-- Simple Query Plan
--
-----------------------------------------------------------------------------------------
SELECT	SalesOrderID
FROM	Sales.SalesOrderHeader H
WHERE	CustomerID = 13440
GO


SELECT
	SalesOrderID,
	ShipDate
FROM	Sales.SalesOrderHeader H
WHERE	CustomerID = 13440
GO
-----------------------------------------------------------------------------------------
--
-- CORRELATED SUBQUERIES
--
-----------------------------------------------------------------------------------------

-- Simple -------------------------------------------
SELECT
	SalesOrderID,
	Quantity = (SELECT SUM(OrderQty) 
				FROM Sales.SalesOrderDetail D
				WHERE SalesOrderID = H.SalesOrderID
				)
FROM	Sales.SalesOrderHeader H
WHERE	CustomerID = 13440
GO

-- Multi-Column ------------------------------------
SELECT
	SalesOrderID,
	Quantity = (SELECT SUM(OrderQty) 
				FROM Sales.SalesOrderDetail D
				WHERE SalesOrderID = H.SalesOrderID
				-- AND SpecialOfferID = 1
				),
	QuantityOffer = (SELECT SUM(OrderQty) 
				FROM Sales.SalesOrderDetail D
			WHERE SalesOrderID = H.SalesOrderID
			AND SpecialOfferID = 1
			)
FROM	Sales.SalesOrderHeader H
WHERE	CustomerID = 13440
GO

-- Single Pass ------------------------------------
SELECT
	H.SalesOrderID,
		SUM(OrderQty) AS Quantity,
		SUM(
			CASE D.SpecialOfferID
			  WHEN 1 THEN OrderQty
			  ELSE 0
			END
		) AS QuantityOffer
FROM	Sales.SalesOrderHeader H
JOIN	Sales.SalesOrderDetail D ON D.SalesOrderID = H.SalesOrderID
WHERE	H.CustomerID = 13440
GROUP BY
	H.SalesOrderID
GO

-- UDF (show reads) -------------------------------- AdventureWorks2008
SELECT
	H.SalesOrderID,
	QuantityOffer0 = dbo.GetSalesOrderQuantity(H.SalesOrderID, 0),
	QuantityOffer1 = dbo.GetSalesOrderQuantity(H.SalesOrderID, 1)
FROM	Sales.SalesOrderHeader H
WHERE	H.CustomerID = 13440
GO



---------------------------------------------------------------------------------------------
-- FUNCTIONS IN THE WHERE CLAUSE
-- Using Functions around an indexed column
-- Two queries, one with functions, one without
---------------------------------------------------------------------------------------------
SELECT	SalesOrderID,
		OrderDate
FROM	Sales.SalesOrderHeader
WHERE	YEAR(OrderDate) = 2003
AND		MONTH(OrderDate) = 7
GO

SELECT	SalesOrderID,
		OrderDate
FROM	Sales.SalesOrderHeader
WHERE	OrderDate BETWEEN '7/1/2003' AND '7/31/2003'
GO


---------------------------------------------------------------------------------------------
-- USING EXISTS
---------------------------------------------------------------------------------------------
DECLARE @Rows INT

SELECT @Rows = COUNT(*)
FROM	Sales.SalesOrderHeader
WHERE	Status = 5

IF @Rows > 0 PRINT 'Found!'
GO

IF EXISTS (SELECT 1 FROM Sales.SalesOrderHeader WHERE	Status = 5)
	PRINT 'Found!'
GO

---------------------------------------------------------------------------------------------
-- Loop performance
---------------------------------------------------------------------------------------------
-- Straight SELECT
SELECT SELECT_Total = SUM(OrderQty * UnitPrice)
FROM	Sales.SalesOrderDetail
WHERE	SalesOrderID = 47018
GO

-- Plain Cursor ------------------------------------
DECLARE @OrderAmount DECIMAL(24,4)
DECLARE @TotalOrders DECIMAL(24,4)
SET @TotalOrders = 0

DECLARE c1 CURSOR
FOR SELECT OrderAmount = OrderQty * UnitPrice
FROM	Sales.SalesOrderDetail
WHERE	SalesOrderID = 47018

OPEN c1

FETCH NEXT FROM c1 INTO @OrderAmount
WHILE (@@fetch_status <> -1)
BEGIN
	IF (@@fetch_status <> -2)
	BEGIN
		SET @TotalOrders = @TotalOrders + @OrderAmount
	END
	FETCH NEXT FROM c1 INTO @OrderAmount
END

CLOSE c1
DEALLOCATE c1

SELECT Plain_Cursor_Total = @TotalOrders
GO

-- READ-ONLY Cursor ----------------------------------
DECLARE @OrderAmount DECIMAL(24,4)
DECLARE @TotalOrders DECIMAL(24,4)
SET @TotalOrders = 0

DECLARE c1 CURSOR
READ_ONLY  -- This makes it read-only
FOR SELECT OrderAmount = OrderQty * UnitPrice
FROM	Sales.SalesOrderDetail
WHERE	SalesOrderID = 47018

OPEN c1

FETCH NEXT FROM c1 INTO @OrderAmount
WHILE (@@fetch_status <> -1)
BEGIN
	IF (@@fetch_status <> -2)
	BEGIN
		SET @TotalOrders = @TotalOrders + @OrderAmount
	END
	FETCH NEXT FROM c1 INTO @OrderAmount
END

CLOSE c1
DEALLOCATE c1

SELECT Readonly_Cursor_Total = @TotalOrders
GO

-- WHILE Loop --------------------------------------------
DECLARE @SalesOrderDetailID INT,
		@NewSalesOrderDetailID INT

DECLARE @OrderAmount DECIMAL(24,4)
DECLARE @TotalOrders DECIMAL(24,4)
SET @TotalOrders = 0

SELECT	TOP 1 @OrderAmount = OrderQty * UnitPrice,
		@SalesOrderDetailID = SalesOrderDetailID,
		@NewSalesOrderDetailID = SalesOrderDetailID
FROM	Sales.SalesOrderDetail
WHERE	SalesOrderID = 47018
ORDER BY SalesOrderDetailID

WHILE (@NewSalesOrderDetailID IS NOT NULL)
BEGIN
	SET @TotalOrders = @TotalOrders + @OrderAmount
	SET @NewSalesOrderDetailID = NULL

	--PRINT @SalesOrderDetailID
	--PRINT @OrderAmount

	SELECT	TOP 1 @OrderAmount = OrderQty * UnitPrice,
			@NewSalesOrderDetailID = SalesOrderDetailID
	FROM	Sales.SalesOrderDetail
	WHERE	SalesOrderID = 47018
	AND		SalesOrderDetailID > @SalesOrderDetailID
	ORDER BY SalesOrderDetailID

	SET @SalesOrderDetailID = @NewSalesOrderDetailID
END


SELECT WhileLoop_Total = @TotalOrders
GO



---------------------------------------------------------------------------------------------
-- NULLS - Handle dynamic parameters in a stored procedure
---------------------------------------------------------------------------------------------
-- Straight SELECT
DECLARE @SalesOrderID INT
SET @SalesOrderID = 43671

SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	SalesOrderID = @SalesOrderID
GO

-- Using COALESCE
DECLARE @SalesOrderID INT
SET @SalesOrderID = 43671
	
SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	SalesOrderID = COALESCE(@SalesOrderID, SalesOrderID)
GO	

-- Using OR @Parm IS NULL
DECLARE @SalesOrderID INT
SET @SalesOrderID = 43671
	
SELECT	*
FROM	Sales.SalesOrderHeader
WHERE	SalesOrderID = @SalesOrderID  OR  @SalesOrderID IS NULL
GO	


-- Nulls & conditional parameters
DECLARE @SalesOrderID INT,
		@SalesOrderDate DATETIME
		
SELECT @SalesOrderID = 43761,
		@SalesOrderDate = NULL  -- '7/3/2006'

DECLARE @SQL NVARCHAR(MAX)

SET @SQL = 'SELECT * FROM Sales.SalesOrderHeader WHERE 1=1 '

IF @SalesOrderID IS NOT NULL
	SET @SQL = @SQL + ' AND	SalesOrderID = @SalesOrderID '

IF @SalesOrderDate IS NOT NULL
	SET @SQL = @SQL + ' AND	OrderDate = @SalesOrderDate '

-- PRINT @SQL

EXEC sp_executesql 
	@SQL, 
	N'@SalesOrderID int, @SalesOrderDate DATETIME', 
	@SalesOrderID, @SalesOrderDate


SELECT
	*
FROM
	Sales.SalesOrderHeader
WHERE  ( SalesOrderID = @SalesOrderID OR  @SalesOrderID IS NULL )
AND		OrderDate = COALESCE(@SalesOrderDAte, OrderDate)
GO
----------------------------------------------------------------------------





IF  EXISTS (SELECT * FROM sys.indexes WHERE object_id = OBJECT_ID(N'[Sales].[SalesOrderHeader]') AND name = N'IX_SalesOrderHeader_OrderDate')
	DROP INDEX [IX_SalesOrderHeader_OrderDate] ON [Sales].[SalesOrderHeader] 
GO