USE
	tempdb;
GO


CREATE SCHEMA
	Inventory;
GO


-- Create the "Inventory.Products" table

CREATE TABLE
	Inventory.Products
(
	Id		INT		NOT NULL	IDENTITY(1,1) ,
	Name		NVARCHAR(50)	NOT NULL ,
	Status		TINYINT		NOT NULL ,
	Weight		DECIMAL(19,2)	NOT NULL ,
	Location	NVARCHAR(20)	NOT NULL ,
	ListPrice	MONEY		NOT NULL ,

	CONSTRAINT
		pk_Products_c_Id
	PRIMARY KEY CLUSTERED
		(Id ASC)
);
GO


-- Populate the "Inventory.Products" table with random data (1,000,000 rows)

INSERT INTO
	Inventory.Products WITH (TABLOCK)
(
	Name ,
	Status ,
	Weight ,
	Location ,
	ListPrice
)
SELECT TOP (1000000)
	Name		= N'Product #' + CAST ((ABS (CHECKSUM (NEWID ())) % 1000000 + 1) AS NVARCHAR(50)) ,
	Status		= ABS (CHECKSUM (NEWID ())) % 10 + 1 ,
	Weight		= CAST ((CAST ((ABS (CHECKSUM (NEWID ())) % 100000 + 1) AS DECIMAL(19,2)) / 100.0) AS DECIMAL(19,2)) ,
	Location	= N'Loc-' + CAST ((ABS (CHECKSUM (NEWID ())) % 999 + 1) AS NVARCHAR(20)) ,
	ListPrice	= CAST ((ABS (CHECKSUM (NEWID ())) % 100000 + 1) AS MONEY) / 100.0
FROM
	sys.all_columns AS T1
CROSS JOIN
	sys.all_columns AS T2;
GO


-- Query the "Inventory.Products" table with an "AND" operator

SELECT
	Id ,
	Name
FROM
	Inventory.Products
WHERE
	Status = 7
AND
	Weight BETWEEN 100.00 AND 200.00
AND
	ListPrice < $2.00;
GO


-- Create a composite index on the "Status", "Weight" and "ListPrice" columns, and include the "Id" and "Name" columns

CREATE NONCLUSTERED INDEX
	ix_Products_nc_nu_Status#Weight#ListPrice_Id#Name
ON
	Inventory.Products
		(
			Status		ASC ,
			Weight		ASC ,
			ListPrice	ASC
		)
INCLUDE
	(Id , Name);
GO


-- Query the "Inventory.Products" table with an "AND" operator

SELECT
	Id ,
	Name
FROM
	Inventory.Products
WHERE
	Status = 7
AND
	Weight BETWEEN 100.00 AND 200.00
AND
	ListPrice < $2.00;
GO


-- Create an index on the "Status" column instead of the composite index created previously

DROP INDEX
	ix_Products_nc_nu_Status#Weight#ListPrice_Id#Name
ON
	Inventory.Products;
GO


CREATE NONCLUSTERED INDEX
	ix_Products_nc_nu_Status
ON
	Inventory.Products (Status ASC);
GO


-- Query the "Inventory.Products" table with an "AND" operator

SELECT
	Id ,
	Name
FROM
	Inventory.Products
WHERE
	Status = 7
AND
	Weight BETWEEN 100.00 AND 200.00
AND
	ListPrice < $2.00;
GO


-- Force the optimizer to use the "ix_Products_nc_nu_Status" index

SELECT
	Id ,
	Name
FROM
	Inventory.Products WITH (INDEX = ix_Products_nc_nu_Status)
WHERE
	Status = 7
AND
	Weight BETWEEN 100.00 AND 200.00
AND
	ListPrice < $2.00;
GO


-- Make the index on the "Status" column a covering index by adding the relevant included columns

DROP INDEX
	ix_Products_nc_nu_Status
ON
	Inventory.Products;
GO


CREATE NONCLUSTERED INDEX
	ix_Products_nc_nu_Status_Weight#ListPrice#Name
ON
	Inventory.Products (Status ASC)
INCLUDE
	(
		Weight ,
		ListPrice ,
		Name
	);
GO


-- Query the "Inventory.Products" table with an "AND" operator

SELECT
	Id ,
	Name
FROM
	Inventory.Products
WHERE
	Status = 7
AND
	Weight BETWEEN 100.00 AND 200.00
AND
	ListPrice < $2.00;
GO


-- Create a (non-covering) index on the "ListPrice" column instead of the index on the "Status" column

DROP INDEX
	ix_Products_nc_nu_Status_Weight#ListPrice#Name
ON
	Inventory.Products;
GO


CREATE NONCLUSTERED INDEX
	ix_Products_nc_nu_ListPrice
ON
	Inventory.Products (ListPrice ASC);
GO


-- Query the "Inventory.Products" table with an "AND" operator

SELECT
	Id ,
	Name
FROM
	Inventory.Products
WHERE
	Status = 7
AND
	Weight BETWEEN 100.00 AND 200.00
AND
	ListPrice < $2.00;
GO


-- Query the "Inventory.Products" table with an "OR" operator

SELECT
	Id ,
	Name
FROM
	Inventory.Products
WHERE
	Status = 7
OR
	Weight BETWEEN 100.00 AND 200.00
OR
	ListPrice < $2.00;
GO


-- Create a (non-covering) index on the "Weight" column

CREATE NONCLUSTERED INDEX
	ix_Products_nc_nu_Weight
ON
	Inventory.Products (Weight ASC);
GO


-- Query the "Inventory.Products" table with an "OR" operator

SELECT
	Id ,
	Name
FROM
	Inventory.Products
WHERE
	Status = 7
OR
	Weight BETWEEN 100.00 AND 200.00
OR
	ListPrice < $2.00;
GO


-- Create a (non-covering) index on the "Status" column

CREATE NONCLUSTERED INDEX
	ix_Products_nc_nu_Status
ON
	Inventory.Products (Status ASC);
GO


-- Query the "Inventory.Products" table with an "OR" operator

SELECT
	Id ,
	Name
FROM
	Inventory.Products
WHERE
	Status = 7
OR
	Weight BETWEEN 100.00 AND 200.00
OR
	ListPrice < $2.00;
GO


-- Add the "Name" column as an included column to all the non-clustered indexes

DROP INDEX
	ix_Products_nc_nu_ListPrice
ON
	Inventory.Products;
GO


DROP INDEX
	ix_Products_nc_nu_Weight
ON
	Inventory.Products;
GO


DROP INDEX
	ix_Products_nc_nu_Status
ON
	Inventory.Products;
GO


CREATE NONCLUSTERED INDEX
	ix_Products_nc_nu_ListPrice_Name
ON
	Inventory.Products (ListPrice ASC)
INCLUDE
	(Name);
GO


CREATE NONCLUSTERED INDEX
	ix_Products_nc_nu_Weight_Name
ON
	Inventory.Products (Weight ASC)
INCLUDE
	(Name);
GO


CREATE NONCLUSTERED INDEX
	ix_Products_nc_nu_Status_Name
ON
	Inventory.Products (Status ASC)
INCLUDE
	(Name);
GO


-- Query the "Inventory.Products" table with an "OR" operator

SELECT
	Id ,
	Name
FROM
	Inventory.Products
WHERE
	Status = 7
OR
	Weight BETWEEN 100.00 AND 200.00
OR
	ListPrice < $2.00;
GO
