-- If the Problem is with BPOOL 
--
-- we can determine the Clerk which is consuming the memory using 
--
-- DBCC memorystatus or sys.dm_os_memory_clerks  and trouble it.
-- 
-- sys.dm_os_memory_clerks can provide a complete picture of SQL Server memory status and can be drilled down using sys.dm_os_memory_objects
-- 
-- Other views which can help to troubleshoot SQL Server memory issues are
 
SELECT * FROM sys.dm_os_memory_objects;
SELECT * FROM sys.dm_os_memory_pools; 
SELECT * FROM sys.dm_os_memory_nodes; 
SELECT * FROM sys.dm_os_memory_cache_entries; 
SELECT * FROM sys.dm_os_memory_cache_hash_tables;
 
-- Few queries which we use to troubleshoot SQL Server memory issues.
-- 
-- Show other information about system memory 
-- 

SELECT total_physical_memory_kb/(1024) AS phys_mem_mb, 
   available_physical_memory_kb/(1024) AS avail_phys_mem_mb, 
   system_cache_kb/(1024) AS sys_cache_mb, 
   (kernel_paged_pool_kb+kernel_nonpaged_pool_kb)/(1024) AS kernel_pool_mb, 
   total_page_file_kb/(1024) AS total_virtual_memory_mb, 
   available_page_file_kb/(1024) AS available_virtual_memory_mb, 
   system_memory_state_desc 
FROM sys.dm_os_sys_memory;
GO 

-- Show memory information about SQLSERVR.EXE process 
-- GetMemoryProcessInfo() API used for this 
-- physical_memory_in_use_kb 

SELECT physical_memory_in_use_kb/(1024) AS sql_physmem_inuse_mb, 
   locked_page_allocations_kb/(1024) AS awe_memory_mb, 
   total_virtual_address_space_kb/(1024) AS max_vas_mb, 
   virtual_address_space_committed_kb/(1024) AS sql_committed_mb, 
   memory_utilization_percentage AS working_set_percentage, 
   virtual_address_space_available_kb/(1024) AS vas_available_mb, 
   process_physical_memory_low AS is_there_external_pressure, 
   process_virtual_memory_low AS is_there_vas_pressure 
FROM sys.dm_os_process_memory;
GO 

SELECT * FROM sys.dm_os_ring_buffers  
WHERE ring_buffer_type like 'RING_BUFFER_RESOURCE%'; 
GO 

SELECT memory_node_id AS node, virtual_address_space_reserved_kb/(1024) as VAS_reserved_mb,
   virtual_address_space_committed_kb/(1024) AS virtual_committed_mb, 
   locked_page_allocations_kb/(1024) AS locked_pages_mb, 
   --single_pages_kb/(1024) AS single_pages_mb, 
   --multi_pages_kb/(1024) AS multi_pages_mb, 
   shared_memory_committed_kb/(1024) AS shared_memory_mb 
FROM sys.dm_os_memory_nodes 
WHERE memory_node_id != 64;
GO 


SELECT size = vadump.size, 
  reserved = SUM(CASE(CONVERT(INT, vadump.base) ^ 0)  WHEN 0 THEN 0 ELSE 1 END), 
  free = SUM(CASE(CONVERT(INT, vadump.base) ^ 0x0) WHEN 0 THEN 1 ELSE 0 END) 
FROM 
   (SELECT CONVERT(VARBINARY, SUM(region_size_in_bytes)) AS size, 
      region_allocation_base_address as base 
   FROM sys.dm_os_virtual_address_dump 
   WHERE region_allocation_base_address <> 0x0 
   GROUP BY region_allocation_base_address 
UNION( 
   SELECT CONVERT(VARBINARY, region_size_in_bytes), 
      region_allocation_base_address 
   FROM sys.dm_os_virtual_address_dump 
   WHERE region_allocation_base_address = 0x0) 
   ) AS vadump 
GROUP BY size;
GO 

 
-- Show stolen pages 
-- 
SELECT (SUM(pages_kb)*1024)/8192 AS total_stolen_pages 
FROM sys.dm_os_memory_clerks;
GO 

-- Breakdown clerks with stolen pages 
SELECT type, name, SUM((pages_kb*1024)/8192) AS stolen_pages 
FROM sys.dm_os_memory_clerks 
WHERE pages_kb > 0 
GROUP BY type, name 
ORDER BY stolen_pages DESC;
GO 

-- Show the total amount of memory consumed by multi_page consumers 
-- 
select SUM(multi_pages_kb)/1024 as total_multi_pages_mb 
from sys.dm_os_memory_clerks 
go 


-- Let's now get the total consumption of virtual allocator 
-- 
SELECT SUM(virtual_memory_committed_kb)/1024 AS total_virtual_mem_mb 
FROM sys.dm_os_memory_clerks;
GO 

-- Breakdown the clerks who use virtual allocator 
-- 
SELECT type, name, sum(virtual_memory_committed_kb)/1024 AS virtual_mem_mb 
FROM sys.dm_os_memory_clerks 
WHERE virtual_memory_committed_kb > 0 
GROUP BY type, name 
ORDER BY virtual_mem_mb DESC;
GO 

-- Is anyone using AWE allocator? 
--
SELECT SUM(awe_allocated_kb)/1024 AS total_awe_allocated_mb 
FROM sys.dm_os_memory_clerks 
GO 

-- Who is the AWE user? 
-- 
SELECT type, name, sum(awe_allocated_kb)/1024 AS awe_allocated_mb 
FROM sys.dm_os_memory_clerks 
WHERE awe_allocated_kb > 0 
GROUP BY type, name 
ORDER BY awe_allocated_mb DESC; 
GO 

-- Does this sync up with what the node thinks? 
-- 
SELECT SUM(virtual_address_space_committed_kb)/1024 AS total_node_virtual_memory_mb, 
   SUM(locked_page_allocations_kb)/1024 AS total_awe_memory_mb, 
   SUM(pages_kb)/1024 AS pages_mb 
FROM sys.dm_os_memory_nodes 
WHERE memory_node_id != 64; 
GO 

-- Total memory used by SQL Server through SQLOS memory nodes 
-- including DAC node 
-- What takes up the rest of the space? 
SELECT (SUM(virtual_address_space_committed_kb)+ 
   SUM(locked_page_allocations_kb)+ 
   SUM(pages_kb))/1024 AS total_sql_memusage_mb 
FROM sys.dm_os_memory_nodes;
GO 


SELECT mc.type, mo.type 
FROM sys.dm_os_memory_clerks mc 
JOIN sys.dm_os_memory_objects mo 
   ON mc.page_allocator_address = mo.page_allocator_address 
GROUP BY mc.type, mo.type 
ORDER BY mc.type, mo.type;
GO