-- Bin Packing algorithm to ensure all shipping bins are fully packed.

-- Original Author: Hugo Kornelius, MVP Deep Dives

-- Set up tables with randomly generated data
IF OBJECT_ID('dbo.Packages', 'U') IS NOT NULL
	BEGIN;
	DROP TABLE dbo.Packages;
	END;
CREATE TABLE dbo.Packages
	(PackageNo int NOT NULL IDENTITY PRIMARY KEY,
	Size smallint NOT NULL,
	BinNo int DEFAULT NULL);
DECLARE @NumPackages int,
	@Loop int;
SET @NumPackages = 100000;
-- Injects further variety into the result sets
SET @NumPackages = @NumPackages + ( SELECT DAY(GETDATE() ) * 2300 )
SET @Loop = 1;
WHILE @Loop <= @NumPackages
	BEGIN;
	INSERT INTO dbo.Packages(Size)
	VALUES (CEILING(RAND() * 30) + CEILING(RAND() * 30));
	SET @Loop = @Loop + 1;
END;
GO

-- Iterative code for bin packing
DECLARE @BinSize smallint
	,@PackageNo int
	,@Size smallint
	,@CurBinNo int
	,@CurSpaceLeft smallint
	,@BinNo int;
SET @BinSize = 100
IF OBJECT_ID('dbo.Bins', 'U') IS NOT NULL
	BEGIN;
	DROP TABLE dbo.Bins;
	END;
CREATE TABLE dbo.Bins
	(BinNo int NOT NULL PRIMARY KEY
	,SpaceLeft smallint NOT NULL);
CREATE INDEX ix_Bins ON dbo.Bins(SpaceLeft);

SET @CurBinNo = 1;
SET @CurSpaceLeft = @BinSize;
INSERT INTO dbo.Bins (BinNo, SpaceLeft)
VALUES (@CurBinNo, @CurSpaceLeft);
DECLARE PackageCursor CURSOR STATIC
	FOR SELECT PackageNo, Size
	FROM dbo.Packages;
OPEN PackageCursor;
FETCH NEXT
FROM PackageCursor
INTO @PackageNo, @Size;
WHILE @@FETCH_STATUS = 0
BEGIN;
	IF @CurSpaceLeft >= @Size
	BEGIN;
		SET @BinNo = @CurBinNo;
	END;
	ELSE
	BEGIN;
		SET @BinNo =
			(SELECT TOP (1) BinNo
			FROM dbo.Bins
			WHERE SpaceLeft >= @Size
			AND BinNo <> @CurBinNo
			ORDER BY SpaceLeft);
	IF @BinNo IS NULL
	BEGIN;
		UPDATE dbo.Bins
		SET SpaceLeft = @CurSpaceLeft
		WHERE BinNo = @CurBinNo;
		SET @CurBinNo = @CurBinNo + 1;
		SET @CurSpaceLeft = @BinSize;
		INSERT INTO dbo.Bins (BinNo, SpaceLeft)
		VALUES (@CurBinNo, @CurSpaceLeft);
		SET @BinNo = @CurBinNo;
		END;
	END;

	UPDATE dbo.Packages
		SET BinNo = @BinNo
		WHERE PackageNo = @PackageNo;
	IF @BinNo = @CurBinNo
	BEGIN;
		SET @CurSpaceLeft = @CurSpaceLeft - @Size;
	END;
	ELSE
	BEGIN;
		UPDATE dbo.Bins
		SET SpaceLeft = SpaceLeft - @Size
		WHERE BinNo = @BinNo;
	END;

	FETCH NEXT
	FROM PackageCursor
	INTO @PackageNo, @Size;
	END;
	IF @CurBinNo IS NOT NULL
	BEGIN;
		UPDATE dbo.Bins
		SET SpaceLeft = @CurSpaceLeft
		WHERE BinNo = @CurBinNo;
	END;
	CLOSE PackageCursor;
	DEALLOCATE PackageCursor;

	SELECT COUNT(*) AS NumPackages,
		SUM(SpaceLeft) AS WastedSpace
		FROM dbo.Bins;
GO

-- Creating the numbers table for use in the set-based bin-packing code
-- using a numbers table.
IF OBJECT_ID('dbo.Numbers', 'U') IS NOT NULL
	BEGIN;
	DROP TABLE dbo.Numbers;
	END;
CREATE TABLE dbo.Numbers
	(Num int NOT NULL PRIMARY KEY);
WITH Digits(d) AS
	(SELECT 0 UNION ALL SELECT 1 UNION ALL SELECT 2 UNION ALL
	SELECT 3 UNION ALL SELECT 4 UNION ALL SELECT 5 UNION ALL
	SELECT 6 UNION ALL SELECT 7 UNION ALL SELECT 8 UNION ALL SELECT 9)
INSERT INTO dbo.Numbers(Num)
SELECT a.d + b.d * 10 + c.d * 100 + d.d * 1000
	+ e.d * 10000 + f.d * 100000 + 1
FROM Digits a, Digits b, Digits c, Digits d, Digits e, Digits f;

-- Set-based iteration for bin packing
DECLARE @BinSize smallint
	,@MinBin int
	,@MaxBin int
	,@BinsNeeded int
	,@Threshold smallint;
SET @BinSize = 100;
SET @MaxBin = 0;
IF OBJECT_ID('dbo.Bins', 'U') IS NOT NULL
	BEGIN;
	DROP TABLE dbo.Bins;
	END;
CREATE TABLE dbo.Bins
	(BinNo int NOT NULL PRIMARY KEY
	,SpaceLeft smallint NOT NULL);
WHILE 1 = 1
BEGIN;
	SET @BinsNeeded =
		(SELECT CEILING(1.0 * SUM(Size) / @BinSize) AS Needed
		FROM dbo.Packages
		WHERE BinNo IS NULL)
	IF @BinsNeeded IS NULL
		BREAK;
	SET @MinBin = @MaxBin + 1;
	SET @MaxBin = @MaxBin + @BinsNeeded;
	INSERT INTO dbo.Bins (BinNo, SpaceLeft)
		SELECT Num, @BinSize
		FROM dbo.Numbers
		WHERE Num BETWEEN @MinBin AND @MaxBin;
	WHILE 1 = 1
	BEGIN;
	SET @Threshold =
		(SELECT MAX(SpaceLeft)
		FROM dbo.Bins
		WHERE BinNo BETWEEN @MinBin AND @MaxBin);
	WITH
		RankedBins AS
			(SELECT BinNo, SpaceLeft,
			ROW_NUMBER() OVER (ORDER BY SpaceLeft DESC) AS Ranking
			FROM dbo.Bins
			WHERE BinNo BETWEEN @MinBin AND @MaxBin
			AND SpaceLeft > 0)
			,RankedPackages AS
		(SELECT Size, BinNo,
			ROW_NUMBER() OVER (ORDER BY Size DESC) AS Ranking
			FROM dbo.Packages
			WHERE BinNo IS NULL
			AND Size <= @Threshold)
	UPDATE p
		SET BinNo = b.BinNo
		FROM RankedPackages AS p
		INNER JOIN RankedBins AS b
		ON b.Ranking = p.Ranking
		WHERE b.SpaceLeft >= p.Size;
	IF @@ROWCOUNT = 0
	BREAK;
		UPDATE dbo.Bins
			SET SpaceLeft = @BinSize -
				(SELECT SUM(p.Size)
				FROM dbo.Packages AS p
				WHERE p.BinNo = Bins.BinNo)
			WHERE BinNo BETWEEN @MinBin AND @MaxBin;
		END;
END;
SELECT COUNT(*) AS NumPackages,
	SUM(SpaceLeft) AS WastedSpace
	FROM dbo.Bins;

BEGIN;
	DROP TABLE dbo.Packages;
	DROP TABLE dbo.Bins;
	DROP TABLE dbo.Numbers;
END;