-- Server Analysis
-- Step #1. Wait Stat Analysis
-- The first step in troubleshooting any instance running SQL Server 2005 
--   or greater is to evaluate wait stats.  Wait stats are always reset at
--   reboot or when intentionally cleared with DBCC SQLPERF('sys.dm_os_wait_stats', CLEAR);
-- Isolate top waits for server instance since last restart or statistics clear
WITH Waits
  AS (SELECT
        wait_type,
        wait_time_ms / 1000. AS wait_time_s,
        100. * wait_time_ms / SUM(wait_time_ms) OVER () AS pct,
        ROW_NUMBER() OVER (ORDER BY wait_time_ms DESC) AS rn
      FROM
        sys.dm_os_wait_stats
      WHERE
        wait_type NOT IN
          ('SLEEP_TASK',
          'BROKER_TASK_STOP',
          'SQLTRACE_BUFFER_FLUSH',
          'CLR_AUTO_EVENT',
          'CLR_MANUAL_EVENT',
          'LAZYWRITER_SLEEP')) -- filter out additional irrelevant waits
SELECT
  W1.wait_type,
  CAST(W1.wait_time_s AS DECIMAL(12, 2)) AS wait_time_s,
  CAST(W1.pct AS DECIMAL(12, 2)) AS pct,
  CAST(SUM(W2.pct) AS DECIMAL(12, 2)) AS running_pct
FROM
  Waits AS W1 INNER JOIN Waits AS W2 ON W2.rn <= W1.rn
GROUP BY
  W1.rn,
  W1.wait_type,
  W1.wait_time_s,
  W1.pct
HAVING
  SUM(W2.pct) - W1.pct < 95; -- percentage threshold

-- Explanation of Common Significant Wait types with BOL explanations
-- 
-- *** Network Related Waits ***
-- ASYNC_NETWORK_IO     
--	Occurs on network writes when the task is blocked behind the network
-- 
-- *** Locking Waits ***
-- LCK_M_IX             
--	Occurs when a task is waiting to acquire an Intent Exclusive (IX) lock
-- LCK_M_IU             
--	Occurs when a task is waiting to acquire an Intent Update (IU) lock
-- LCK_M_S              
--	Occurs when a task is waiting to acquire a Shared lock
--
-- *** I/O Related Waits ***
-- ASYNC_IO_COMPLETION  
--	Occurs when a task is waiting for I/Os to finish
-- IO_COMPLETION        
--	Occurs while waiting for I/O operations to complete. This wait type generally represents 
-- 	non-data page I/Os. Data page I/O completion waits appear as PAGEIOLATCH_* waits
-- PAGEIOLATCH_SH       
--	Occurs when a task is waiting on a latch for a buffer that is in an I/O request. The 
--	latch request is in Shared mode. Long waits may indicate problems in the disk subsystem.
-- PAGEIOLATCH_EX       
--	Occurs when a task is waiting on a latch for a buffer that is in an I/O request. The 
--	latch request is in Exclusive mode. Long waits may indicate problems in the disk system.
-- WRITELOG             
--	Occurs while waiting for a log flush to complete. Common operations that cause log 
--	flushes are checkpoints and transaction commits.
-- PAGELATCH_EX         
--	Occurs when a task is waiting on a latch for a buffer that is not in an I/O request. 
--	The latch request is in Exclusive mode.
-- BACKUPIO             
--	Occurs when a backup task is waiting for data, or is waiting for a buffer in which to 
--	store data
--
-- *** CPU Related Waits ***
-- SOS_SCHEDULER_YIELD  
--	Occurs when a task voluntarily yields the scheduler for other tasks to execute. During 
--	this wait the task is waiting for its quantum to be renewed.
-- THREADPOOL           
--	Occurs when a task is waiting for a worker to run on. This can indicate that the maximum 
--	worker setting is too low, or that batch executions are taking unusually long, thus 
--	reducing the number of workers available to satisfy other batches.
-- CX_PACKET
--	Occurs when trying to synchronize the query processor exchange iterator. You may consider 
--	lowering the degree of parallelism if contention on this wait type becomes a problem.


--Step #2. Transaction Log Write Analysis
SELECT CAST(([num_of_bytes_written] / 1048576.0) AS NUMERIC(10,2)) AS [MBs]
FROM sys.dm_io_virtual_file_stats(DB_ID('SalesAudit'),2) ;

--Step #3. Latency by File
SELECT DB_NAME(database_id) AS 'Database Name',
  file_id,
  io_stall_read_ms / num_of_reads AS 'Avg Read Transfer/ms',
  io_stall_write_ms / num_of_writes AS 'Avg Write Transfer/ms'
FROM sys.dm_io_virtual_file_stats(-1, -1)
WHERE num_of_reads > 0
  AND num_of_writes > 0;

--Also correlated with this for IO stalls and the next query for IO by Database:
--Average stalls per read, write and total. The 1.0 is to avoid division by zero errors. The 
--columns are arranged to provide summary info on the left, then more granular data further 
--to the right.  IO stalls are a good indication of disk IO pressure.

SELECT
  DB_NAME(database_id) AS "Database",
  FILE_NAME(file_id) AS "File",
    CAST( (io_stall_read_ms + io_stall_write_ms) /
    (1.0 + num_of_reads + num_of_writes) AS NUMERIC(10, 1))
    AS 'avg_io_stall_ms',
  (io_stall_read_ms + io_stall_write_ms) AS io_stalls,
  num_of_reads + num_of_writes AS total_io,
  io_stall_read_ms,
  io_stall_write_ms,
  num_of_reads,
  num_of_writes,
  CAST(io_stall_read_ms / (1.0 + num_of_reads) AS NUMERIC(10, 1))
    AS 'avg_read_stall_ms',
  CAST(io_stall_write_ms / (1.0 + num_of_writes) AS NUMERIC(10, 1))
    AS 'avg_write_stall_ms'
FROM sys.dm_io_virtual_file_stats(NULL, NULL)
ORDER BY avg_io_stall_ms DESC;

--Calculating the Total I/O for Each Database
SELECT
  DB_NAME(database_id) AS database_name,
  CAST(SUM(num_of_bytes_read + num_of_bytes_written) / 1048576. AS DECIMAL(12, 2)) AS io_in_mb,
  CAST(SUM(num_of_bytes_read) / 1048576. AS DECIMAL(12, 2)) AS reads_in_mb,
  CAST(SUM(num_of_bytes_written) / 1048576. AS DECIMAL(12, 2)) AS writes_in_mb,
  SUM(num_of_reads) AS 'Number of Read',
  SUM(num_of_writes) AS 'Number of Writes' 
FROM sys.dm_io_virtual_file_stats(NULL, NULL) AS DM_IO_Stats
GROUP BY database_id
ORDER BY io_in_mb DESC;

--Step #4. Memory Utilization

-- Memory Clerk Usage across the SQL Server 2008+ instance
-- You will almost always see CACHESTORE_SQLCP at the top of the list, 
-- since so many applications (such as SharePoint) love to use ad-hoc queries.
SELECT   TOP (20) type, name, SUM(pages_kb)
FROM   sys.dm_os_memory_clerks
GROUP BY [type], [name]
ORDER BY SUM(pages_kb) DESC;


--Step #5. Compiles:Recompiles Ratio
declare @single_use_counts int, @multi_use_counts int;
with PlanCacheCte as 
(
    select
        db_name(st.dbid) as database_name,
        cp.bucketid,
        cp.usecounts,
        cp.size_in_bytes,
        cp.objtype,
        st.text
    from sys.dm_exec_cached_plans cp
    cross apply sys.dm_exec_sql_text(cp.plan_handle) st
    where cp.cacheobjtype = 'Compiled Plan'
)
select @single_use_counts = count(*)
from PlanCacheCte
where usecounts = 1;

with PlanCacheCte as 
(
    select
        db_name(st.dbid) as database_name,
        cp.bucketid,
        cp.usecounts,
        cp.size_in_bytes,
        cp.objtype,
        st.text
    from sys.dm_exec_cached_plans cp
    cross apply sys.dm_exec_sql_text(cp.plan_handle) st
    where cp.cacheobjtype = 'Compiled Plan'
)
select @multi_use_counts = count(*)
from PlanCacheCte
where usecounts > 1

select
    @single_use_counts as single_use_counts,
    @multi_use_counts as multi_use_counts,
    @single_use_counts * 1.0 / (@single_use_counts + @multi_use_counts) * 100
        as percent_single_use_counts

--Number of invocations for all cached plans
SELECT TOP 50
    qs.plan_generation_num,
    qs.execution_count,
    qs.statement_start_offset,
    qs.statement_end_offset,
    st.text
    FROM sys.dm_exec_query_stats qs
    CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) st
    WHERE qs.plan_generation_num > 1
    ORDER BY qs.plan_generation_num DESC;

--PLAN CACHE ANALYSIS

-- SQL Script #1: Query Plan Reuse
-- Query plan reuse (refer to http://msdn.microsoft.com/en-us/library/ee343986.aspx)
SELECT b.[cacheobjtype], b.[objtype], b.[usecounts],
  a.[dbid], a.[objectid], b.[size_in_bytes], a.[text]
FROM sys.dm_exec_cached_plans as b
CROSS APPLY sys.dm_exec_sql_text(b.[plan_handle]) AS a
ORDER BY [usecounts] DESC;



-- SQL Script #2: Query IO
-- we want reusable code, absence of ad hoc SQL
-- we want relatively few rows with low usecounts
SELECT TOP 25
  (qs.total_logical_reads + qs.total_logical_writes) / qs.execution_count AS [Avg IO],
  SUBSTRING(qt.text, qs.statement_start_offset / 2, 
(CASE WHEN qs.statement_end_offset = -1 
THEN LEN(CONVERT(NVARCHAR(MAX), qt.text)) * 2   ELSE qs.statement_end_offset END - qs.statement_start_offset) / 2) AS query_text,
   qt.dbid,
   qt.objectid
FROM     sys.dm_exec_query_stats AS qs 
  CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) AS qt
ORDER BY [Avg IO] DESC;
  


-- SQL Script #3: Top Query Resource Consumers  
-- find out query statistics for those queries that are currently in the plan cache.  
-- Note this is for queries in particular. It shows resource consumption for SELECT
-- statements individually. So this isnt the one to use for stored procedures and 
-- functions since it shows the queries that are executed within those objects, not
-- the objects themselves.
SELECT  TOP 25 cp.usecounts AS [execution_count],
        qs.total_worker_time AS CPU,
        qs.total_elapsed_time AS ELAPSED_TIME,
        qs.total_logical_reads AS LOGICAL_READS,
        qs.total_logical_writes AS LOGICAL_WRITES,
        qs.total_physical_reads AS PHYSICAL_READS,
        SUBSTRING(text, 
          CASE WHEN statement_start_offset = 0 OR statement_start_offset IS NULL
            THEN 1 
            ELSE statement_start_offset/2 + 1 
            END,                    
          CASE WHEN statement_end_offset = 0 OR statement_end_offset = -1
            OR statement_end_offset IS NULL 
            THEN LEN(text) 
            ELSE statement_end_offset/2 
            END - 
          CASE WHEN statement_start_offset = 0 OR statement_start_offset IS NULL 
            THEN 1 
            ELSE statement_start_offset/2  END + 1  )  
        AS [Statement]        
FROM sys.dm_exec_query_stats qs     
JOIN sys.dm_exec_cached_plans cp on qs.plan_handle = cp.plan_handle    
CROSS APPLY sys.dm_exec_sql_text(cp.plan_handle) st
ORDER BY qs.total_logical_reads DESC;  



-- SQL Script #4: Top Procedure Resource Consumers  
-- Similar to the earlier script but shows proc and UDF objects.  Also, much 
-- more flexible in terms of which metric to evaluate performance upon.  Change the 
-- commented ORDER BY clauses to sort by the most important key performance metric.
SELECT   TOP 25
  -- the following four columns are NULL for ad hoc and prepared batches
  DB_Name(qp.dbid) AS dbname,
  qp.dbid,
  qp.objectid,
  qp.number, 
  -- qp.query_plan, --the query plan can be *very* useful; enable if desired
  qt.text,
  SUBSTRING(qt.text, (qs.statement_start_offset / 2) + 1, ((CASE statement_end_offset WHEN -1 THEN DATALENGTH(qt.text) ELSE qs.statement_end_offset  
END - qs.statement_start_offset) / 2) + 1) AS statement_text,
  qs.creation_time,
  qs.last_execution_time,
  qs.execution_count,
  qs.total_worker_time / qs.execution_count AS avg_worker_time,
  qs.total_physical_reads / qs.execution_count AS avg_physical_reads,
  qs.total_logical_reads / qs.execution_count AS avg_logical_reads,
  qs.total_logical_writes / qs.execution_count AS avg_logical_writes,
  qs.total_elapsed_time / qs.execution_count AS avg_elapsed_time,
  qs.total_clr_time / qs.execution_count AS avg_clr_time,
  qs.total_worker_time,
  qs.last_worker_time,
  qs.min_worker_time,
  qs.max_worker_time,
  qs.total_physical_reads,
  qs.last_physical_reads,
  qs.min_physical_reads,
  qs.max_physical_reads,
  qs.total_logical_reads,
  qs.last_logical_reads,
  qs.min_logical_reads,
  qs.max_logical_reads,
  qs.total_logical_writes,
  qs.last_logical_writes,
  qs.min_logical_writes,
  qs.max_logical_writes,
  qs.total_elapsed_time,
  qs.last_elapsed_time,
  qs.min_elapsed_time,
  qs.max_elapsed_time,
  qs.total_clr_time,
  qs.last_clr_time,
  qs.min_clr_time,
  qs.max_clr_time, 
  --, qs.sql_handle , qs.statement_start_offset , qs.statement_end_offset
  qs.plan_generation_num -- , qp.encrypted
FROM sys.dm_exec_query_stats AS qs
  CROSS APPLY sys.dm_exec_query_plan(qs.plan_handle) AS qp
  CROSS APPLY sys.dm_exec_sql_text(qs.sql_handle) AS qt
--WHERE...
ORDER BY qs.total_worker_time DESC --CPU

--ORDER BY qs.execution_count      DESC  --Frequency
--ORDER BY qs.total_elapsed_time   DESC  --Durn
--ORDER BY qs.total_logical_reads  DESC  --Reads
--ORDER BY qs.total_logical_writes DESC  --Writes
--ORDER BY qs.total_physical_reads DESC  --PhysicalReads
--ORDER BY avg_worker_time         DESC  --AvgCPU
--ORDER BY avg_elapsed_time        DESC  --AvgDurn
--ORDER BY avg_logical_reads       DESC  --AvgReads
--ORDER BY avg_logical_writes      DESC  --AvgWrites
--ORDER BY avg_physical_reads      DESC  --AvgPhysicalReads

--sample WHERE clauses
--WHERE last_execution_time > '20070507 15:00'
--WHERE execution_count = 1
--WHERE SUBSTRING(qt.text, (qs.statement_start_offset/2) + 1,
--    ((CASE statement_end_offset
--        WHEN -1 THEN DATALENGTH(qt.text)
--        ELSE qs.statement_end_offset END
--            - qs.statement_start_offset)/2) + 1)
--      LIKE '%MyText%'
/* Original Author: Jimmy May, Microsoft jimmymay@microsoft.com
   aspiringgeek@live.com
   http://blogs.msdn.com/jimmymay
   Note: modify WHERE & ORDER BY clauses to suit circumstances */


-- SQL Script #5: Implicit Conversions in the Plan Cache  
-- Implicit conversions are a major anti-pattern that degrades performance.
SET TRANSACTION ISOLATION LEVEL READ UNCOMMITTED

DECLARE @dbname SYSNAME
SET @dbname = QUOTENAME(DB_NAME());

WITH XMLNAMESPACES
   (DEFAULT 'http://schemas.microsoft.com/sqlserver/2004/07/showplan')

SELECT stmt.value('(@StatementText)[1]', 'varchar(max)'),
       t.value('(ScalarOperator/Identifier/ColumnReference/@Schema)[1]', 'varchar(128)'),
       t.value('(ScalarOperator/Identifier/ColumnReference/@Table)[1]', 'varchar(128)'),
       t.value('(ScalarOperator/Identifier/ColumnReference/@Column)[1]', 'varchar(128)'),
       ic.DATA_TYPE AS ConvertFrom,
       ic.CHARACTER_MAXIMUM_LENGTH AS ConvertFromLength,
       t.value('(@DataType)[1]', 'varchar(128)') AS ConvertTo,
       t.value('(@Length)[1]', 'int') AS ConvertToLength,
       query_plan
FROM   sys.dm_exec_cached_plans AS cp
       CROSS APPLY sys.dm_exec_query_plan(plan_handle) AS qp
       CROSS APPLY query_plan.nodes('/ShowPlanXML/BatchSequence/Batch/Statements/StmtSimple') AS batch(stmt)
       CROSS APPLY stmt.nodes('.//Convert[@Implicit="1"]') AS n(t)
       JOIN INFORMATION_SCHEMA.COLUMNS AS ic
         ON QUOTENAME(ic.TABLE_SCHEMA) = t.value('(ScalarOperator/Identifier/ColumnReference/@Schema)[1]', 'varchar(128)') AND
            QUOTENAME(ic.TABLE_NAME) = t.value('(ScalarOperator/Identifier/ColumnReference/@Table)[1]', 'varchar(128)')
            AND ic.COLUMN_NAME = t.value('(ScalarOperator/Identifier/ColumnReference/@Column)[1]', 'varchar(128)')
WHERE  t.exist('ScalarOperator/Identifier/ColumnReference[@Database=sql:variable("@dbname")][@Schema!="[sys]"]') = 1;

-- SQL Script #6: Objects in the Cache  
-- Objects by Database, can take a long time to run on big systems with > 64gb of RAM
SELECT  CASE database_id
          WHEN 32767 THEN 'ResourceDb'
          ELSE DB_NAME(database_id)
        END AS database_name ,
        COUNT(*) AS cached_pages_count ,
        COUNT(*) * .0078125 AS cached_megabytes 
/* Each page is 8kb, which is .0078125 of an MB */
FROM    sys.dm_os_buffer_descriptors
GROUP BY DB_NAME(database_id) ,
        database_id
ORDER BY cached_pages_count DESC ;

 
