-- =============================================
-- Author:		Lara Rubbelke
-- Create date: November 8, 2010
-- Description:	DBCC SHOW_STATISTICS sample procedure.
--   Originally for use with the Separation of Duties Framework
-- =============================================

DECLARE @DatabaseName sysname 
	, @schema_name sysname
	, @object_name sysname
	, @target sysname
	, @STAT_HEADER bit = 0
	, @DENSITY_VECTOR bit = 0
	, @HISTOGRAM bit = 0

SELECT @DatabaseName = 'AdventureWorks2014'
		, @schema_name  = 'Production'
		, @object_name = 'Product'
		, @target = 'PK_Product_ProductID'
		, @STAT_HEADER = 1
		, @DENSITY_VECTOR = 1
		, @HISTOGRAM = 1

SET NOCOUNT ON
/*********************************************************************************
    Name:       dba.sysproc_DBCC_Show_Statistics
 
    Purpose:    DBCC SHOW_STATISTICS displays current query optimization 
				statistics for a table or indexed view. 
				
      @DatabaseName	Required. The name of the source database.

      @schema_name	Required. The name of the source schema.
      
      @object_name	Required. The name of the source table.

      @target	Required. Name of the index, statistics, or column for 
				which to display statistics information. If target is a 
				name of an existing index or statistics on a table or 
				indexed view, the statistics information about this target 
				is returned. If target is the name of an existing column, and 
				an automatically created statistics on this column exists, 
				information about that auto-created statistic is returned. 
				If an automatically created statistic does not exist for a 
				column target, error message 2767 is returned.
      
      @STAT_HEADER	Valid values 1 (set option on) and 0. Default 0. 
				Set to 1 to display the Stat_Header information.
      
      @DENSITY_VECTOR	Valid values 1 (set option on) and 0. Default 0. 
				Set to 1 to display the Density_Vector information.
      
      @HISTOGRAM	Valid values 1 (set option on) and 0. Default 0. 
				Set to 1 to display the Histogram information.
*********************************************************************************/
	
	IF (@DatabaseName IS NULL)
	BEGIN
		DECLARE @ErrorMessage NVARCHAR(4000);
		DECLARE @ErrorSeverity INT;
		DECLARE @ErrorState INT;

		SET @ErrorMessage = N'Procedure or function sysproc_DBCC_Show_Statistics expects parameter @DatabaseName, which was not supplied.';
		SET @ErrorSeverity = 16;
		SET @ErrorState = 1;

		RAISERROR (@ErrorMessage, @ErrorSeverity, @ErrorState )
		RETURN;		
	END
	ELSE
	DECLARE @cmd nvarchar(max), @cmdparam nvarchar(max)
	
	SET @cmd = 'DBCC Show_Statistics( "' 
			+ quotename(@DatabaseName) 
			+ '.' + QUOTENAME(@Schema_Name) 
			+ '.' + QUOTENAME(@Object_Name) 
			+ '", ' + QUOTENAME(@target) + ')'
	
	SET @cmdparam = 			+ (
			SELECT CASE WHEN @STAT_HEADER = 1 OR @DENSITY_VECTOR = 1 OR @HISTOGRAM = 1 
				THEN ' WITH ' 
				END
			+ (SELECT STUFF((SELECT ',' + INPUT 
			FROM (
			SELECT CASE WHEN @STAT_HEADER = 1 THEN 'STAT_HEADER' END AS INPUT
			UNION
			SELECT CASE WHEN @DENSITY_VECTOR = 1 THEN 'DENSITY_VECTOR' END 
			UNION
			SELECT CASE WHEN @HISTOGRAM = 1 THEN 'HISTOGRAM' END
			) P
			FOR XML PATH('')), 1, 1, '')))
	
	SET @cmd = @cmd + isnull(@cmdparam, '')

	EXEC sp_executesql @cmd
GO
