/*
Examples:
--Return index information for a specific index.
@DatabaseName = 'AdventureWorks2014', @schema_name = 'HumanResources', @object_name = 'Department', @index_or_statistics_name = 'AK_Department_Name', @mode = 'DEFAULT', @OrderByColumn = 'avg_fragmentation_in_percent', @SortOrder = 'DESC'

-- Return index information for all indexes in AdventureWorks.
@DatabaseName = 'AdventureWorks2014', @mode = 'SAMPLED',
@OrderByColumn = 'avg_fragmentation_in_percent', @SortOrder = 'DESC'
GO
*/
DECLARE @DatabaseName sysname 
	, @schema_name sysname = NULL
	, @object_name sysname = NULL
	, @index_or_statistics_name sysname = NULL
	, @partition_number int = NULL
	, @mode sysname = 'DEFAULT'
	, @OrderByColumn sysname = 'avg_fragmentation_in_percent'
	, @SortOrder sysname = 'DESC'

SELECT @DatabaseName = 'AdventureWorks2014' 
	, @schema_name = NULL
	, @object_name = NULL
	, @index_or_statistics_name = NULL
	, @partition_number = NULL
	, @mode = 'DEFAULT'  --DEFAULT, NULL, LIMITED (the default), SAMPLED, or DETAILED
	, @OrderByColumn = 'avg_fragmentation_in_percent'
	, @SortOrder = 'DESC'

SET NOCOUNT ON
/*********************************************************************************
	--By Lara Rubbelke, from the Separation of Duties White Paper
	--Create date: November 8, 2010 

    Name:       originally, sysproc_dm_db_index_physical_stats
 
    Purpose:    Issues the sys.dm_db_index_physical_stats function to return
				index information.

      @DatabaseName	Required. The name of the source database. 

      @schema_name	Optional. The name of the source schema. Specify NULL 
				to return information for all tables and views in 
				the specified database.  
      
      @object_name	Optional. The name of the source table. Specify NULL 
				to return information for all tables and views in 
				the specified database. 
	  
	  @index_or_statistics_name	Optional. The name of the index. 
				Specify NULL to return information for all 
				indexes for a base table or view.  
	  
	  @partition_number	Optional. The partition number in the object.
				partition_number is int. Specify NULL to return 
				information for all partitions of the owning object.

	  @mode		Optional.  Is the name of the mode. mode specifies the 
				scan level that is used to obtain statistics. mode 
				is sysname. Valid inputs are DEFAULT, NULL, LIMITED, 
				SAMPLED, or DETAILED. The default (NULL) is LIMITED.
				
	  @OrderByColumn Optional. Name of column to order the results. Valid 
				values are avg_fragmentation_in_percent, page_count, 
				avg_fragment_size_in_pages, avg_page_space_used_in_percent, 
				record_count. Default avg_fragmentation_in_percent.
				
	  @SortOrder	Optional. Valid values are ASC and DESC. Default DESC.
*********************************************************************************/

BEGIN
	DECLARE @index_id INT
	DECLARE @ErrorMessage NVARCHAR(4000);
	DECLARE @ErrorSeverity INT;
	DECLARE @ErrorState INT;
		
	IF(ISNULL(@mode, 'DEFAULT') NOT IN ('DEFAULT', 'LIMITED', 'SAMPLED', 'DETAILED'))
	BEGIN 
		SET @ErrorMessage = N'Procedure sysproc_dm_db_index_physical_stats expects parameter @mode of DEFAULT, LIMITED, SAMPLED OR DETAILED. Please supply a valid value.';
		SET @ErrorSeverity = 16;
		SET @ErrorState = 1;
		RAISERROR (@ErrorMessage, @ErrorSeverity, @ErrorState )
		RETURN;	
	END
	
	IF (@DatabaseName IS NULL)
	BEGIN
		SET @ErrorMessage = N'Procedure or function sysproc_dm_db_index_physical_stats expects parameter @DatabaseName, which was not supplied.';
		SET @ErrorSeverity = 16;
		SET @ErrorState = 1;

		RAISERROR (@ErrorMessage, @ErrorSeverity, @ErrorState )
		RETURN;		
	END

	IF(ISNULL(@OrderByColumn, 'avg_fragmentation_in_percent') NOT IN ('avg_fragmentation_in_percent', 'page_count', 'avg_fragment_size_in_pages', 'avg_page_space_used_in_percent', 'record_count'))
	BEGIN 
		SET @ErrorMessage = N'Procedure sysproc_dm_db_index_physical_stats expects parameter @OrderByColumn of avg_fragmentation_in_percent, page_count, avg_fragment_size_in_pages, avg_page_space_used_in_percent, record_count. Please supply a valid value.';
		SET @ErrorSeverity = 16;
		SET @ErrorState = 1;
		RAISERROR (@ErrorMessage, @ErrorSeverity, @ErrorState )
		RETURN;	
	END
	
		IF(ISNULL(@SortOrder, 'DESC') NOT IN ('DESC', 'ASC'))
	BEGIN 
		SET @ErrorMessage = N'Procedure sysproc_dm_db_index_physical_stats expects parameter @SortOrder of DESC or ASC. Please supply a valid value.';
		SET @ErrorSeverity = 16;
		SET @ErrorState = 1;
		RAISERROR (@ErrorMessage, @ErrorSeverity, @ErrorState )
		RETURN;	
	END
		
	SET @mode = CASE WHEN ISNULL(@mode, 'DEFAULT') = 'DEFAULT' 
		THEN 'LIMITED' 
		ELSE @mode END
	
	IF (@index_or_statistics_name IS NOT NULL)
	BEGIN
		DECLARE @indexcmd nvarchar(500);
		DECLARE @name sysname;
		DECLARE @ParmDefinition nvarchar(256);
		DECLARE @index_idout int;
		
		SET @indexcmd = N'(SELECT @index_idout = index_id FROM ' + QUOTENAME(@DatabaseName) + '.sys.indexes WHERE name = @name)'
		SET @ParmDefinition = N'@name sysname, @index_idout int OUTPUT';

		EXECUTE sp_executesql @indexcmd, @ParmDefinition, @name = @index_or_statistics_name, @index_idout=@index_id OUTPUT;
	END
	
	IF @object_name IS NOT NULL 
	BEGIN
		SET @object_name = @DatabaseName + '.' + ISNULL(@schema_name, '') + '.' + @object_name 

		DECLARE @cmd nvarchar(max)
		SET @cmd = '
		SELECT DB_NAME(ips.database_id) AS DatabaseName
			, OBJECT_NAME(ips.object_id, ips.database_id) AS ObjectName
			, i.name AS index_name
			, *
		FROM sys.dm_db_index_physical_stats(DB_ID(' + QUOTENAME(@DatabaseName, '''') + '),OBJECT_ID(' + QUOTENAME(@object_name, '''') + '), ' + ISNULL(CAST(@index_id AS nvarchar(10)), 'NULL') + ', NULL, ' + QUOTENAME(@mode, '''') + ')AS ips
		INNER JOIN ' + QUOTENAME(@DatabaseName) + '.sys.indexes i
		ON ips.object_id = i.object_id AND ips.index_id = i.index_id
		ORDER BY ' + QUOTENAME(@OrderByColumn) + ' ' + @SortOrder + ';' 

PRINT @CMD
		EXEC sp_executesql @cmd;
	END
	ELSE 
	BEGIN
		SET @cmd = '
		SELECT DB_NAME(ips.database_id) AS DatabaseName
			, OBJECT_NAME(ips.object_id, ips.database_id) AS ObjectName
			, i.name AS index_name
			, *
		FROM sys.dm_db_index_physical_stats(DB_ID(' + QUOTENAME(@DatabaseName, '''') + '),NULL, NULL, NULL, ' + QUOTENAME(@mode, '''') + ')AS ips
		INNER JOIN ' + QUOTENAME(@DatabaseName) + '.sys.indexes i
		ON ips.object_id = i.object_id AND ips.index_id = i.index_id
		ORDER BY ' + QUOTENAME(@OrderByColumn) + ' ' + @SortOrder + ';' 

		EXEC sp_executesql @cmd;
		END
END
GO