USE AdventureWorksDWDenali;
GO

SET STATISTICS IO ON;
-- Display actual execution plan

/****************************************************
Compare performance of coverning index to columnstore
****************************************************/

-- Query 1
-- This should use the covering index
SELECT S.ResellerKey,
	SUM(S.ExtendedAmount) AS Total
	FROM dbo.FactResellerSalesPtnd AS S
	JOIN dbo.DimReseller AS R
		ON S.ResellerKey = R.ResellerKey
	WHERE R.ProductLine = 'Road'
	GROUP BY S.ResellerKey
	OPTION (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX); -- 422K logical reads using covering index
GO

-- Query 2
SELECT S.ResellerKey,
	SUM(S.ExtendedAmount) AS Total, count(*)
	FROM dbo.FactResellerSalesPtnd AS S -- WITH (INDEX (csindx_FactResellerSalesPtnd))
	JOIN dbo.DimReseller AS R
		ON S.ResellerKey = R.ResellerKey
	WHERE R.ProductLine = 'Road'
	GROUP BY S.ResellerKey; -- 5725 logical reads
GO

/****************************************************
Look at the three new catalog tables for columnstore
****************************************************/

-- Query set 3
SELECT * FROM sys.column_store_index_stats
	WHERE [name] = 'csindx_FactResellerSalesPtnd';
SELECT * FROM sys.column_store_segments
	order by hobt_id, column_id;
SELECT * FROM sys.column_store_dictionaries
	order by hobt_id, column_id;

/****************************************************
Compare sizes of rowstore vs. columnstore indexes
****************************************************/

-- Query 4
-- determine total size of columnstore indexes
SELECT CAST(SUM(on_disk_size_MB) AS numeric(10,2)) AS TotalSizeInMB
FROM
(
   (SELECT SUM(css.on_disk_size)/(1024.0*1024.0) on_disk_size_MB
    FROM sys.indexes AS i
    JOIN sys.partitions AS p
        ON i.object_id = p.object_id 
    JOIN sys.column_store_segments AS css
        ON css.hobt_id = p.hobt_id
    WHERE i.object_id = object_id('FactResellerSalesPtnd') 
    AND i.type_desc = 'NONCLUSTERED COLUMNSTORE') 
  UNION ALL
   (SELECT SUM(csd.on_disk_size)/(1024.0*1024.0) on_disk_size_MB
    FROM sys.indexes AS i
    JOIN sys.partitions AS p
        ON i.object_id = p.object_id 
    JOIN sys.column_store_dictionaries AS csd
        ON csd.hobt_id = p.hobt_id
    WHERE i.object_id = object_id('FactResellerSalesPtnd') 
    AND i.type_desc = 'NONCLUSTERED COLUMNSTORE') 
) AS SegmentsPlusDictionary
GO

-- query to show size of covering index
DECLARE @OBJECT_NAME VARCHAR(255) = 'FactResellerSalesPtnd';
 DECLARE @temp TABLE ( indexID  BIGINT,
	objectId  BIGINT,
	index_name NVARCHAR(MAX),
	used_page_count  BIGINT,
	pages  BIGINT);

--Insert into temp table
INSERT INTO @temp
SELECT P.index_id,
	P.OBJECT_ID ,
	I.name,
	SUM (used_page_count),
	SUM (CASE
		WHEN (p.index_id < 2) THEN (in_row_data_page_count + lob_used_page_count + row_overflow_used_page_count)
		ELSE lob_used_page_count + row_overflow_used_page_count
	END)
	FROM sys.dm_db_partition_stats AS P
	JOIN sys.indexes AS I
		ON I.index_id = P.index_id
		AND I.OBJECT_ID = P.OBJECT_ID
	WHERE p.OBJECT_ID = OBJECT_ID(@OBJECT_NAME)
	AND I.[name] = 'XIE2_FactResellerSalesPtnd_Covering'
	GROUP BY P.index_id, I.Name, P.OBJECT_ID;

SELECT index_name INDEX_NAME,
	LTRIM (STR((CASE WHEN used_page_count > pages THEN (used_page_count - pages) ELSE 0 END) * 8 / 1024, 15, 0) + ' MB') AS INDEX_SIZE
	FROM @temp T;
GO

/****************************************************
Batch Execution Mode - not used on UNIONs
Workaround
****************************************************/

-- Query 5
-- Show difference between BATCH processing and ROW processing
-- First ROW - BATCH processing not supported with OUTER JOINs
SELECT P.EnglishPromotionName, -- 14 sec
	COALESCE(SUM(F.ExtendedAmount), 0) AS TotalSales
	FROM dbo.DimPromotion AS P
	LEFT OUTER JOIN dbo.FactResellerSalesPtnd AS F
		ON P.PromotionKey = F.PromotionKey
		AND F.OrderDateKey >= 20070101
	GROUP BY P.EnglishPromotionName;

-- Query 6
-- BATCH processing - this is an INNER JOIN - different results than above, but a reasonable comparison
SELECT P.EnglishPromotionName,
	COALESCE(SUM(F.ExtendedAmount), 0) AS TotalSales
	FROM dbo.DimPromotion AS P
	JOIN dbo.FactResellerSalesPtnd AS F
		ON P.PromotionKey = F.PromotionKey
		AND F.OrderDateKey >= 20070101
	GROUP BY P.EnglishPromotionName;

-- Query 7
-- Workaround to use INNER JOIN on large data set, then OUTER JOIN on much smaller
WITH GroupedSales AS (SELECT P.PromotionKey,
	COALESCE(SUM(F.ExtendedAmount), 0) AS TotalSales
	FROM dbo.DimPromotion AS P
	JOIN dbo.FactResellerSalesPtnd AS F
		ON P.PromotionKey = F.PromotionKey
		AND F.OrderDateKey >= 20070101
	GROUP BY P.PromotionKey
)SELECT P.EnglishPromotionName,
	COALESCE(G.TotalSales, 0)
	FROM dbo.DimPromotion AS P
	LEFT OUTER JOIN GroupedSales AS G
		ON P.PromotionKey = G.PromotionKey
	ORDER BY P.EnglishPromotionName;

/*******************************************
Bait & Switch - nightly single partition update
*******************************************/

-- 1. drop columnstore on stage1
DROP INDEX csindx_FactResellerSalesStage1 ON dbo.FactResellerSalesStage1;

-- 2. truncate stage1
TRUNCATE TABLE dbo.FactResellerSalesStage1;

-- 3. create columnstore on stage1
CREATE NONCLUSTERED COLUMNSTORE INDEX csindx_FactResellerSalesStage1 ON dbo.FactResellerSalesStage1
(
	ProductKey,
	OrderDateKey,
	DueDateKey,
	ShipDateKey,
	ResellerKey,
	EmployeeKey,
	PromotionKey,
	CurrencyKey,
	SalesTerritoryKey,
	SalesOrderNumber,
	SalesOrderLineNumber,
	RevisionNumber,
	OrderQuantity,
	UnitPrice,
	ExtendedAmount,
	UnitPriceDiscountPct,
	DiscountAmount,
	ProductStandardCost,
	TotalProductCost,
	SalesAmount,
	TaxAmt,
	Freight,
	CarrierTrackingNumber,
	CustomerPONumber,
	OrderDate,
	DueDate,
	ShipDate
) ON [PRIMARY];

-- 4. switch Fact partition to stage1
ALTER TABLE dbo.FactResellerSalesPtnd
	SWITCH PARTITION 20 TO dbo.FactResellerSalesStage1;

-- 5. disable columnstore on stage1
ALTER INDEX csindx_FactResellerSalesStage1 ON dbo.FactResellerSalesStage1 DISABLE;

-- 6. modify stage1
UPDATE dbo.FactResellerSalesStage1
	SET CarrierTrackingNumber = 'VERN-4FBF-B6'
	WHERE CarrierTrackingNumber = '1E90-4FBF-B6';

-- 7. enable columnstore on stage1
ALTER INDEX csindx_FactResellerSalesStage1 ON dbo.FactResellerSalesStage1 REBUILD;

-- 8. switch stage1 to Fact partition
ALTER TABLE dbo.FactResellerSalesStage1
	SWITCH TO dbo.FactResellerSalesPtnd PARTITION 20;

/*******************************************
Trickle Load - live updates
*******************************************/

-- 1. drop columnstore index on stage2
IF EXISTS(SELECT * FROM sys.indexes
	WHERE [name] = 'csindx_FactResellerSalesStage2')
BEGIN;
	DROP INDEX csindx_FactResellerSalesStage2 ON dbo.FactResellerSalesStage2;
END;

-- 2. empty out Stage2 
TRUNCATE TABLE dbo.FactResellerSalesStage2;

-- 3. copy partition to Stage2 -- ~30 seconds
INSERT INTO dbo.FactResellerSalesStage2
	SELECT * FROM dbo.FactResellerSalesPtnd
	WHERE OrderDateKey >= 20010701
	AND OrderDateKey < 20010801;

-- 4. update stage2
UPDATE dbo.FactResellerSalesStage2
	SET CarrierTrackingNumber = '1E90-4FBF-B6'
	WHERE CarrierTrackingNumber = 'VERN-4FBF-B6';

-- 5. drop columnstore index on stage1
DROP INDEX csindx_FactResellerSalesStage1 ON dbo.FactResellerSalesStage1;

-- 6. truncate stage1
TRUNCATE TABLE dbo.FactResellerSalesStage1;

-- 7. create columnstore index on stage1
CREATE NONCLUSTERED COLUMNSTORE INDEX csindx_FactResellerSalesStage1 ON dbo.FactResellerSalesStage1
(
	ProductKey,
	OrderDateKey,
	DueDateKey,
	ShipDateKey,
	ResellerKey,
	EmployeeKey,
	PromotionKey,
	CurrencyKey,
	SalesTerritoryKey,
	SalesOrderNumber,
	SalesOrderLineNumber,
	RevisionNumber,
	OrderQuantity,
	UnitPrice,
	ExtendedAmount,
	UnitPriceDiscountPct,
	DiscountAmount,
	ProductStandardCost,
	TotalProductCost,
	SalesAmount,
	TaxAmt,
	Freight,
	CarrierTrackingNumber,
	CustomerPONumber,
	OrderDate,
	DueDate,
	ShipDate
) ON [PRIMARY];

-- 8. create columnstore index on stage2
CREATE NONCLUSTERED COLUMNSTORE INDEX csindx_FactResellerSalesStage2 ON dbo.FactResellerSalesStage2
(
	ProductKey,
	OrderDateKey,
	DueDateKey,
	ShipDateKey,
	ResellerKey,
	EmployeeKey,
	PromotionKey,
	CurrencyKey,
	SalesTerritoryKey,
	SalesOrderNumber,
	SalesOrderLineNumber,
	RevisionNumber,
	OrderQuantity,
	UnitPrice,
	ExtendedAmount,
	UnitPriceDiscountPct,
	DiscountAmount,
	ProductStandardCost,
	TotalProductCost,
	SalesAmount,
	TaxAmt,
	Freight,
	CarrierTrackingNumber,
	CustomerPONumber,
	OrderDate,
	DueDate,
	ShipDate
) ON [PRIMARY];

BEGIN TRAN;
-- 9. switch partition to stage1
ALTER TABLE dbo.FactResellerSalesPtnd
	SWITCH PARTITION 20 TO dbo.FactResellerSalesStage1;

-- 10. switch stage2 to partition
ALTER TABLE dbo.FactResellerSalesStage2
	SWITCH TO dbo.FactResellerSalesPtnd PARTITION 20;
COMMIT;
	
-- 11. drop columnstore index on stage2
DROP INDEX csindx_FactResellerSalesStage2 ON dbo.FactResellerSalesStage2;

-- 12. copy Fact partition to stage2
INSERT INTO dbo.FactResellerSalesStage2
	SELECT * FROM dbo.FactResellerSalesPtnd
	WHERE OrderDateKey >= 20010701
	AND OrderDateKey < 20010801;

-- 13. GOTO step 4.
