/*
T-SQL Batch to provoke a high number of forwarded records.

Inspired by material by Kalen Delaney (http://sqlblog.com/blogs/kalen_delaney/archive/2009/11/11/fragmentation-and-forwarded-records-in-a-heap.aspx)
and Sankar Reddy (http://sankarreddy.com/2010/03/how-can-i-tell-if-a-sql-server-system-is-affected-by-forwarded-records/).

Author: Kevin Kline
*/

--USE AdventureWorks;
--GO

IF EXISTS (SELECT 1 FROM sys.tables
            WHERE name = 'Address2' AND schema_id =1)
        DROP TABLE dbo.Address2;
GO

IF EXISTS (SELECT 1 FROM sys.tables
            WHERE name = 'Details' AND schema_id =1)
        DROP TABLE dbo.Details;
GO

SELECT *, CONVERT(VARCHAR(500), 'comments') AS comments 
INTO dbo.Address2 
FROM Person.Address;
GO

/*
Check Details Block, use this section for each table, before and after: Address2 and Details.
Before, note that the pages are almost full and there are no forwarded records.
After, there will be thousands of forwarded records.

SELECT index_type_desc, page_count, avg_page_space_used_in_percent,
     avg_record_size_in_bytes,forwarded_record_count
FROM sys.dm_db_index_physical_stats(db_id('AdventureWorks'), 
       object_id('Address2'),null, null, 'detailed');
*/

-- Provoke forwarded records.
UPDATE dbo.Address2
SET comments = replicate('a', 500);
GO

-- Let's see what happens when we read every row in the table, 10 times.
DECLARE @counter1 tinyint
DECLARE @counter2 tinyint

SELECT @counter1 = 0, @counter2 = 0

WHILE @counter1 <> 10
BEGIN
  SELECT @counter1 = (@counter1 + 1)

  SELECT * FROM dbo.Address2
END
GO

-- Let's do it with an even bigger table. I'll use regular INSERT this time.
CREATE TABLE dbo.Details 
    ([SalesOrderID] [int] NOT NULL, 
    [SalesOrderDetailID] [int] NOT NULL, 
    [CarrierTrackingNumber] [nvarchar](25) NULL, 
    [OrderQty] [smallint] NOT NULL, 
    [ProductID] [int] NOT NULL, 
    [SpecialOfferID] [int] NOT NULL, 
    [UnitPrice] [money] NOT NULL, 
    [UnitPriceDiscount] [money] NOT NULL); 
GO    
INSERT INTO dbo.Details 
SELECT [SalesOrderID] 
      ,[SalesOrderDetailID] 
      ,[CarrierTrackingNumber] 
      ,[OrderQty] 
      ,[ProductID] 
      ,[SpecialOfferID] 
      ,[UnitPrice] 
      ,[UnitPriceDiscount] 
  FROM [AdventureWorks].[Sales].[SalesOrderDetail]; 
GO 

-- A table scan takes as many reads as there are pages, i.e. 856 in this case 
SELECT * FROM dbo.Details; 
GO 

-- Provoke forwarded records.
 
ALTER TABLE dbo.Details 
ADD notes CHAR(100); 
GO 
 
UPDATE dbo.Details 
SET notes = 'notes'; 
GO

DECLARE @counter1 tinyint
DECLARE @counter2 tinyint

SELECT @counter1 = 0, @counter2 = 0

WHILE @counter1 <> 10
BEGIN
  SELECT @counter1 = (@counter1 + 1)

  SELECT * FROM Details
END
GO
